import FreeCAD as App
import FreeCADGui as Gui

from PySide.QtCore import QTimer
from pickle import TRUE

class TechDrawExtensions:
	views_to_repaint = {}
	
	def __init__(self):
		workbench = Gui.getWorkbench("AssemblyHandbookWorkbench") #: :type workbench: AssemblyHandbookWorkbench
		workbench.docObserver.onObjectTypeSelected('view_annotate_balloon_selected', 'TechDraw::DrawViewBalloon', lambda operation, obj, sub, point: self.onBalloonSelected(operation, obj, sub, point))
	
	def repaint(self, view):
		self.views_to_repaint[view] = True
		QTimer.singleShot(100, self._do_repaint)
		
	def _do_repaint(self):
		import time
		
		workbench = Gui.getWorkbench("AssemblyHandbookWorkbench") #: :type workbench: AssemblyHandbookWorkbench
		
		to_repaint = self.views_to_repaint.keys()
		self.views_to_repaint = {}
		
		for view in to_repaint:
			print("Repainting " + view.Name)
			
			selected_balloons = []
			for obj in Gui.Selection.getSelection():
				if obj.TypeId == 'TechDraw::DrawViewBalloon' and obj.SourceView == view and 'AssemblyHandbook_PartName' in obj.PropertiesList:
					selected_balloons.append(obj)
					
			is_first_part = True
			
			if len(selected_balloons) == 0:
				selected_balloons.append(None)
			
			for balloon in selected_balloons:
				if balloon is not None:
					doc = balloon.Document
					partLink = doc.getObject(balloon.AssemblyHandbook_PartName)
					
					part_view = workbench.partsCache.getPart2DView(view, partLink)
					
					center = self.computePartCenter(view, partLink)
				
				# iterate edges of actual view and highlight matching edges
				for edgeIdx in range(10000):
					hasEdge = False
					try:
						edge = view.getEdgeByIndex(edgeIdx)
						hasEdge = True
					except:
						pass
					if not hasEdge:
						break
					
					# reset edge format
					if is_first_part:
						view.formatGeometricEdge(edgeIdx,1,0.25,0,True)
					
					if balloon is not None and (not hasattr(edge.Curve, 'Degree') or edge.Curve.Degree == 1):
						edgeData = [
							edge.Vertexes[0].X - center.x,
							edge.Vertexes[0].Y - center.y,
							edge.Vertexes[1].X - center.x,
							edge.Vertexes[1].Y - center.y
						]
						v0 = App.Vector(edgeData[0], edgeData[1])
						v1 = App.Vector(edgeData[2], edgeData[3])
						
						for line in part_view.cached_lines:
							l0 = App.Vector(line[0], line[1])
							l1 = App.Vector(line[2], line[3])
							#d = abs(edgeData[0] - line[0]) + abs(edgeData[1] - line[1]) + abs(edgeData[2] - line[2]) + abs(edgeData[3] - line[3])
							d = v0.distanceToLineSegment(l0, l1).Length + v1.distanceToLineSegment(l0, l1).Length
							if d < 0.001:
								view.formatGeometricEdge(edgeIdx,1,0.25,(0,0.85,0),True)
				
				is_first_part = False
			
			view.requestPaint()
	
	def onBalloonSelected(self, operation, balloon, sub, point):
		import time
		
		#print(operation, obj.Name, sub, point)
		if "AssemblyHandbook_PartName" in balloon.PropertiesList:
			#print(operation + " " + balloon.Name)
			view = balloon.SourceView
			self.repaint(view)
				
	def getViewPage(self, view):
		for obj in view.InList:
			if obj.TypeId == 'TechDraw::DrawPage':
				if view in obj.Views:
					return  obj
		return None
	
	def computePartCenter(self, view, obj):
		if obj.TypeId == 'App::Link':
			partLink = obj
			objectCenterWorld = partLink.LinkPlacement.Matrix.multiply(partLink.LinkedObject.Shape.CenterOfGravity)
		else:
			objectCenterWorld = obj.Shape.CenterOfGravity
		
		vertId = view.makeCosmeticVertex3d(objectCenterWorld)
		vert = view.getCosmeticVertex(vertId)
		objectCenterView = vert.Point
		view.removeCosmeticVertex(vertId)
		return objectCenterView
